package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"io"
	"os"
	"os/exec"
)

var Version = "develop"

func main() {
	fs := flag.NewFlagSet("imp", flag.ExitOnError)
	writeFlag := fs.Bool("write", false, "Write the re-ordered imports instead of just printing them")
	fs.BoolVar(writeFlag, "w", *writeFlag, "--write")
	ignoreFlag := fs.String("ignore", ".impignore", "Path to .impignore file")
	fs.StringVar(ignoreFlag, "i", *ignoreFlag, "--ignore")
	gofumptFlag := fs.Bool("gofumpt", false, "Also format using gofumpt")
	gofumptExtraFlag := fs.Bool("gofumpt-extra", false, "Also use extra rules with gofumpt")
	stdinFlag := fs.Bool("stdin", false, "Format stdin")

	if err := fs.Parse(os.Args[1:]); err != nil {
		fmt.Println(err)
	}

	if *stdinFlag {
		mod, err := modInfo()
		if err != nil {
			fmt.Println(err)
			return
		}
		src, err := io.ReadAll(os.Stdin)
		if err != nil {
			fmt.Println(err)
			return
		}
		formatted, err := impFormat(src, mod, *gofumptFlag, *gofumptExtraFlag)
		if err != nil {
			fmt.Println(err)
		}
		fmt.Print(string(formatted))
	}

	root := "."
	if fs.NArg() > 0 {
		root = fs.Arg(0)
	}

	if err := runImp(root, *ignoreFlag, *writeFlag, *gofumptFlag, *gofumptExtraFlag); err != nil {
		fmt.Println(err)
	}
}

type module struct {
	Path      string
	GoVersion string
}

func (m *module) UnmarshalJSON(data []byte) error {
	type mod struct {
		Module struct {
			Path string
		}
		Go string
	}
	var mo mod
	if err := json.Unmarshal(data, &mo); err != nil {
		return err
	}
	*m = module{
		Path:      mo.Module.Path,
		GoVersion: mo.Go,
	}
	return nil
}

func modInfo() (module, error) {
	cmd := exec.Command("go", "mod", "edit", "--json")
	out, err := cmd.Output()
	if err != nil {
		return module{}, err
	}

	var m module
	if err := json.Unmarshal(out, &m); err != nil {
		return m, err
	}
	return m, nil
}
